import {
  useReducer,
  useState,
  createContext,
  useCallback,
  useContext,
  useMemo,
  useEffect,
} from 'react';
import cn from 'classnames';
import { motion } from 'framer-motion';
import IconArrowDown from '../../../../public/images/icons/icon-arrow-down.svg';

const dropdownVariants = {
  open: { display: 'block' },
  closed: { display: 'none' },
};
const dropdownState = {
  option: undefined,
  selected: undefined,
  onSelect: () => {},
};
const dropdownReducer = (state, action) => {
  switch (action.type) {
    case 'SET_OPTION':
      return (state = {
        ...state,
        option: action.payload,
      });
    case 'SET_SELECTED':
      return (state = {
        ...state,
        selected: action.payload,
      });
    default:
      return state;
  }
};

const DropdownContext = createContext({
  state: dropdownState,
  dispatch: () => {},
});

const DropdownItem = (props) => {
  const { label, value } = props;

  const { option, selected, onSelect } = useContext(DropdownContext).state;
  const dispatch = useContext(DropdownContext).dispatch;

  const handleClick = useCallback(() => {
    onSelect({ label, value });
    dispatch({ type: 'SET_OPTION', payload: { label, value } });
  }, [dispatch, onSelect, label, value]);

  useEffect(() => {
    if (selected && selected === value) {
      dispatch({ type: 'SET_OPTION', payload: { label, value } });
    }
  }, [dispatch, selected, label, value]);

  return (
    <li
      className={`px-4 py-2 w-full hover:bg-gray-300 cursor-pointer ${
        (selected === value && !option?.value) ||
        (option?.value && option?.value === value)
          ? 'bg-gray-300'
          : 'bg-white'
      }`}
      onClick={handleClick}
    >
      {label}
    </li>
  );
};
const Dropdown = (props) => {
  const {
    label,
    info,
    error,
    children,
    selected,
    onHide,
    placeholder,
    menuClassName,
    menuPosition = 'fixed',
    onSelect,
    variant = 'filled',
  } = props;

  const [show, setShow] = useState(false);
  const [state, dispatch] = useReducer(dropdownReducer, { dropdownState });

  const labelClass = cn({
    'inline-block mb-1 text-gray-500': true,
    '!text-red-500': error,
  });
  const infoClass = cn({
    'text-[12px] text-gray-400 mt-1': true,
    '!text-red-500': error,
  });
  const buttonClass = cn({
    'flex items-center': true,
    'border bg-white px-4 py-2 w-full mb-4': variant === 'filled',
    'border-gray-300': !error,
    'border-red-500': error,
  });
  const buttonTextClass = cn({
    'leading-6': true,
    'text-red-300': error,
  });
  const menuClass = cn(
    {
      'absolute z-20 bg-white border border-t-0 border-gray-300': true,
      'left-0': menuPosition === 'left',
      'right-0': menuPosition === 'right',
      'right-0 left-0': menuPosition === 'fixed',
    },
    menuClassName
  );

  const handleOnSelect = useCallback(
    (option) => {
      if (onSelect) {
        onSelect(option);
      }
    },
    [onSelect]
  );
  const handleOnToggle = useCallback(() => {
    setShow(!show);
  }, [show]);
  const handleOnHide = useCallback(() => {
    setShow(false);
    if (onHide && !selected) {
      onHide();
    }
  }, [onHide, selected]);

  const renderValue = useMemo(
    () => ({
      state: {
        ...state,
        onSelect: handleOnSelect,
      },
      dispatch,
    }),
    [state, dispatch, handleOnSelect]
  );
  const renderLabel = (() => {
    if (selected) {
      return state.option?.label;
    }
    return placeholder || 'Choose Option';
  })();

  useEffect(() => {
    dispatch({ type: 'SET_SELECTED', payload: selected });
  }, [dispatch, selected]);
  useEffect(() => {
    if (show) {
      window.addEventListener('click', handleOnHide);
    }
    return () => window.removeEventListener('click', handleOnHide);
  }, [show]); // eslint-disable-line

  return (
    <DropdownContext.Provider value={renderValue}>
      <div className="w-full">
        {label && (
          <label className={labelClass} htmlFor="">
            {label}
          </label>
        )}

        <div className="relative z-1">
          <button
            type="button"
            className={buttonClass}
            onClick={handleOnToggle}
          >
            <span className={buttonTextClass}>{renderLabel}</span>
            <span className="ml-auto">
              <IconArrowDown />
            </span>
          </button>
          <motion.ul
            animate={show ? 'open' : 'closed'}
            initial={false}
            variants={dropdownVariants}
            className={menuClass}
          >
            {children}
          </motion.ul>
        </div>
        {info && <div className={infoClass}>{info}</div>}
      </div>
    </DropdownContext.Provider>
  );
};

export default Object.assign(Dropdown, {
  Item: DropdownItem,
});
