import { useState, useEffect, useCallback, useMemo } from 'react';
import Link from 'next/link';
import { useRouter } from 'next/router';
import { useTranslation } from 'next-i18next';
import { motion, AnimatePresence } from 'framer-motion';
import { useAppProvider } from '@/context/AppProvider';
import CartToggler from '@/containers/CartContainer/partials/CartToggler';
import { dummyCart } from '@/data/cart';
import InputSearch from './partials/InputSearch';
import { backdropVariant } from '@/app/constant/variant';
import SearchItem from './partials/SearchItem';
import Dropdown from '@/components/common/Dropdown';
import Logo from '../../../../public/images/logo.svg';
import IconMenu from '../../../../public/images/icons/icon-menu.svg';
import IconAccount from '../../../../public/images/icons/icon-account.svg';
import IconClose from '../../../../public/images/icons/icon-close.svg';
import IconIg from '../../../../public/images/icons/icon-instagram.svg';
import IconFb from '../../../../public/images/icons/icon-facebook.svg';
import IconTw from '../../../../public/images/icons/icon-twitter.svg';
import IconTt from '../../../../public/images/icons/icon-tiktok.svg';

import { cartGetters } from '@react-test-storefront/medusa';
import RegionSelect from './partials/RegionSelect';

const navbarScrollVariant = {
  scroll: {
    y: '-130px',
  },
  default: {
    y: '0px',
  },
};
const navMobileMenuVariant = {
  open: {
    display: 'block',
    y: '0%',
    opacity: 1,
    transition: {
      duration: 0.5,
    },
  },
  close: {
    y: '-20px',
    opacity: 0,
    transition: {
      duration: 0.1,
    },
    transitionEnd: {
      display: 'none',
    },
  },
};
// const searchBoxVariant = {
//   show: {
//     y: 0,
//     opacity: 1,
//     display: 'block',
//   },
//   hide: {
//     y: -30,
//     opacity: 0,
//     transitionEnd: {
//       display: 'none',
//     },
//   },
// };

const DesktopMenuItem = ({ link, label, active }) => (
  <Link href={link} passHref>
    <a className="inline-block mx-6 text-sm tracking-widest uppercase group">
      <span
        className={`
          transition-all group-hover:text-black 
          ${active ? 'text-black' : 'text-gray-500'}
        `}
      >
        {label}
      </span>
      <span
        className={`
          block h-[1px] mt-2 bg-black group-hover:w-1/2 transition-all 
          ${active ? 'w-1/2 ' : 'w-0'}
        `}
      />
    </a>
  </Link>
);
const MobileMenuItem = ({ link, label, active }) => (
  <Link href={link} passHref>
    <a className="block py-5 mx-6 text-sm tracking-widest text-center uppercase group">
      <span
        className={`
          transition-all group-hover:text-black 
          ${active ? 'text-black' : 'text-gray-500'}
        `}
      >
        {label}
      </span>
    </a>
  </Link>
);
const SocialMediaItem = ({ link, children }) => (
  <Link href={link} passHref>
    <a className="block w-6 h-6 mx-2 text-gray-500 transition-colors fill-current hover:text-black">
      {children}
    </a>
  </Link>
);

let lastScrollTop = 0;
const Navbar = () => {
  const { cartObject: { cart } } = useAppProvider();
  const cartTotal = useMemo(() => cartGetters.getTotalItems(cart), [cart]);

  /* helper */
  const router = useRouter();
  const { t } = useTranslation();

  /* state */
  const [toggleMenu, setToggleMenu] = useState(false);
  const [toggleSearch, setToggleSearch] = useState(false);
  const [isScrolling, setIsScrolling] = useState(false);

  /* handler */
  const handleToggleMenu = useCallback(() => {
    setToggleMenu(!toggleMenu);
  }, [toggleMenu]);
  const handleToggleSearch = useCallback(() => {
    setToggleSearch(!toggleSearch);
  }, [toggleSearch]);
  const handleWindowScroll = useCallback(() => {
    const scrollTop = window.pageYOffset || document.documentElement.scrollTop;

    if (scrollTop >= lastScrollTop && scrollTop >= 130) {
      setIsScrolling(true);
    } else {
      setIsScrolling(false);
    }

    lastScrollTop = scrollTop <= 0 ? 0 : scrollTop;
  }, []);
  const handleChangeLocale = useCallback(
    (locale = 'en') => {
      router.push(router.asPath, undefined, { locale });
    },
    [router]
  );

  /* effect */
  useEffect(() => {
    if (toggleMenu) {
      document.body.classList.add('navbar-menu-show');
    }
    if (toggleSearch) {
      document.body.classList.add('has-backdrop');
      window.addEventListener('click', handleToggleSearch);
    }

    return () => {
      window.removeEventListener('click', handleToggleSearch);
      document.body.classList.remove('has-backdrop');
      document.body.classList.remove('navbar-menu-show');
    };
  }, [toggleMenu, toggleSearch]); // eslint-disable-line

  useEffect(() => {
    window.addEventListener('scroll', handleWindowScroll);

    return () => {
      window.removeEventListener('scroll', handleWindowScroll);
    };
  }, []); // eslint-disable-line

  return (
    <>
      {/* backdrop search */}
      <div className="hidden lg:block">
        <motion.div
          key="backdrop-search"
          variants={backdropVariant}
          animate={toggleSearch ? 'show' : 'hide'}
          initial={false}
          className="fixed top-0 bottom-0 left-0 right-0 bg-black bg-opacity-30 z-[109]"
        />
      </div>
      <motion.div
        variants={navbarScrollVariant}
        animate={isScrolling ? 'scroll' : 'default'}
        transition={{ ease: 'linear' }}
        initial={false}
        className="fixed top-0 z-[110] w-[100vw]"
      >
        <div className="relative z-[110] bg-gray-200 h-[45px]">
          <div className="container h-full mx-auto">
            <div className="flex h-full items-center">
              <div className="flex-auto">
                <a href="" className="text-[14px]">
                  Help & Faqs
                </a>
              </div>
              <div className="flex-initial">
                <Dropdown
                  variant="text"
                  selected={router.locale}
                  menuPosition="right"
                  onSelect={(option) => handleChangeLocale(option?.value)}
                >
                  <Dropdown.Item label="Indonesia" value="id" />
                  <Dropdown.Item label="English" value="en" />
                </Dropdown>
              </div>
            </div>
          </div>
        </div>
        <div className="Navbar bg-white lg:h-[85px]">
          <div className="container h-full mx-auto">
            {/* Mobile Nav */}
            <div className="grid items-center grid-cols-5 lg:hidden">
              <div className="col-span-12 relative z-[90] h-[85px] grid grid-cols-5 items-center bg-white">
                <div className="col-span-1">
                  <button onClick={handleToggleMenu}>
                    {toggleMenu ? <IconClose /> : <IconMenu />}
                  </button>
                </div>
                <div className="flex justify-center col-span-3">
                  <Link href="/">
                    <a className="flex w-[100px]">
                      <Logo />
                    </a>
                  </Link>
                </div>
                <div className="flex justify-end col-span-1 gap-x-1">
                  <Link href="/account/my-profile" passHref>
                    <a>
                      <IconAccount />
                    </a>
                  </Link>

                  <CartToggler />
                </div>
              </div>

              <AnimatePresence>
                {toggleMenu && (
                  <motion.div
                    animate={{ y: '-0%' }}
                    className="fixed top-0 bottom-0 left-0 right-0 pt-[130px] h-[100vh] bg-white pointer-events-none z-10 overflow-x-hidden overflow-y-auto"
                    exit={{ y: '-100%' }}
                    initial={{ y: '-100%' }}
                    transition={{ ease: 'linear', duration: 0.3 }}
                  >
                    <div className="container bg-white">
                      <div className="flex flex-col w-full h-full pointer-events-auto relative">
                        {/* Nav search */}
                        <InputSearch
                          viewType="mobile"
                          className="w-full"
                          onClick={(e) => {
                            setToggleSearch(true);
                            e.stopPropagation();
                          }}
                          onClose={() => handleToggleSearch()}
                        />
                        {/* Nav Menu */}
                        <motion.div
                          variants={navMobileMenuVariant}
                          animate={toggleSearch ? 'close' : 'open'}
                          initial={true}
                        >
                          <nav>
                            <MobileMenuItem
                              active={router.pathname === '/'}
                              label={t('header:menu_home')}
                              link="/"
                            />
                            <MobileMenuItem
                              active={router.pathname === '/store'}
                              label={t('header:menu_shop')}
                              link="/store"
                            />
                            <MobileMenuItem
                              active={router.pathname === 'men'}
                              label={t('header:menu_men')}
                              link="/"
                            />
                            <MobileMenuItem
                              active={router.pathname === 'women'}
                              label={t('header:menu_women')}
                              link="/"
                            />
                          </nav>
                          <div className="flex self-center justify-around mb-8 mt-11">
                            <SocialMediaItem link="#">
                              <IconFb />
                            </SocialMediaItem>
                            <SocialMediaItem link="#">
                              <IconIg />
                            </SocialMediaItem>
                            <SocialMediaItem link="#">
                              <IconTw />
                            </SocialMediaItem>
                            <SocialMediaItem link="#">
                              <IconTt />
                            </SocialMediaItem>
                          </div>
                        </motion.div>
                        {/* Nav Menu Search */}
                        <motion.div
                          variants={navMobileMenuVariant}
                          animate={toggleSearch ? 'open' : 'close'}
                          initial={false}
                          className="absolute w-full right-0 pb-[80px] mb-10 top-[55px]"
                        >
                          <SearchItem
                            type="keyword"
                            href="/store"
                            label="Search Keyword"
                          />

                          <hr className="my-4 w-[95%] mx-auto" />
                          <div className="flex flex-col gap-[15px]">
                            {dummyCart.map((item) => (
                              <SearchItem
                                key={item.path}
                                type="item"
                                href={`/store/${item.path}`}
                                imgSrc={item.image}
                                imgAlt={item.name}
                                label={item.name}
                              />
                            ))}
                          </div>
                        </motion.div>
                      </div>
                    </div>
                  </motion.div>
                )}
              </AnimatePresence>
            </div>

            {/* Desktop Nav */}
            <div className="lg:flex gap-[30px] w-full h-full items-center hidden">
              <div className="flex-initial">
                <Link href="/">
                  <a className="flex w-[100px]">
                    <Logo />
                  </a>
                </Link>
              </div>
              <div className="flex items-center flex-auto">
                <nav className="flex items-center">
                  <DesktopMenuItem
                    active={router.pathname === '/'}
                    label={t('header:menu_home')}
                    link="/"
                  />
                  <DesktopMenuItem
                    active={router.pathname === '/store'}
                    label={t('header:menu_shop')}
                    link="/store"
                  />
                </nav>
                {/* Input Search */}
                {/*
                <div className="relative">
                  <InputSearch
                    onClick={(e) => {
                      setToggleSearch(true);
                      e.stopPropagation();
                    }}
                  />
                  <motion.div
                    variants={searchBoxVariant}
                    animate={toggleSearch ? 'show' : 'hide'}
                    initial={false}
                    transition={{ ease: 'linear' }}
                    className="absolute left-0 right-0 top-[70px] min-h-[320px] max-h-[440px] overflow-x-hidden overflow-y-auto bg-white z-[111] drop-shadow-xl p-6"
                  >
                    <SearchItem
                      type="keyword"
                      href="/store"
                      label="Search Keyword"
                    />

                    <hr className="my-4 w-[95%] mx-auto" />
                    <div className="flex flex-col gap-[15px]">
                      {dummyProducts.map((item) => (
                        <SearchItem
                          key={item.path}
                          type="item"
                          href={`/store/${item.path}`}
                          imgSrc={item.image}
                          imgAlt={item.name}
                          label={item.name}
                        />
                      ))}
                    </div>
                  </motion.div>
                </div> */}
              </div>
              <div className="flex-initial flex items-center gap-x-5">
                {/* TODO Temporary remove untill we can provide implementation */}
                {/* <Link href="/account/my-profile" passHref>
                  <a>
                    <IconAccount />
                  </a>
                </Link> */}
                <RegionSelect />
                {cartTotal}
                <CartToggler />
              </div>
            </div>
          </div>
        </div>
      </motion.div>
    </>
  );
};

export default Navbar;
