import { useMemo } from 'react';
import cn from 'classnames';
import Link from 'next/link';
import Image from 'next/image';

import { cartGetters } from '@react-test-storefront/medusa';
import Quantity from '@/components/sections/Quantity';
import { useAppProvider } from '@/context/AppProvider';
import { formatAmountWithSymbol } from '@/utils/price';

const placeholderImg = '/product-img-placeholder.svg';

const CartItem = ({ item, variant = 'base' }) => {
  const { cartObject: { cart, cartFunctions } } = useAppProvider();
  const { updateItemQty, removeItem } = cartFunctions();

  // GETTERS
  const { getItemPrice, getItemName, getItemImage, getItemQty, getItemSlug, getItemInventory } = cartGetters;
  const itemPrice = useMemo(() => getItemPrice(item), [item]);
  const itemName = useMemo(() => getItemName(item), [item]);
  const itemImage = useMemo(() => getItemImage(item), [item]);
  const itemQty = useMemo(() => getItemQty(item), [item, cart]);
  const itemSlug = useMemo(() => getItemSlug(item), [item]);
  const itemInventory = useMemo(() => getItemInventory(item), [item]);

  const handleChangeQuantity = (n = 1) => {
    const val = Number(itemQty) + n;
    updateItemQty({ currentCart: cart, product: item, quantity: val });
  };
  const handleChange = async ({ target: { value } }) => {
    const qty = Number(value);
    updateItemQty({ currentCart: cart, product: item, quantity: qty });
  };

  const handleRemove = () => {
    removeItem({ currentCart: cart, product: item });
  };

  const rootClass = cn({
    ['px-5 mb-4']: variant === 'base',
    ['px-0 mb-4']: variant === 'advanced'
  });

  const thumbClass = cn({
    'relative z-0 overflow-hidden cursor-pointer flex-shrink-0 mb-2 md:mb-0': true,
    ['w-full md:w-16 h-[240px] md:h-16']: variant === 'base',
    ['w-full md:w-36 h-[240px] md:h-36']: variant === 'advanced',
  });

  const titleClass = cn({
    ['font-normal']: variant === 'base',
    ['font-semibold']: variant === 'advanced',
  });

  const priceClass = cn({
    ['font-normal']: variant === 'base',
    ['font-semibold']: variant === 'advanced',
  });

  // const metaClass = cn('text-xs text-gray-500 block mb-1');

  return (
    <li className={`CartItem ${rootClass}`}>
      <div className="flex flex-col md:flex-row w-full overflow-hidden py-4 gap-x-4">
        <div className={thumbClass}>
          <Link href={`/store/${itemSlug}`}>
            <a>
              <Image
                src={itemImage || placeholderImg}
                alt={`${itemName}-cover` || 'Product Image'}
                layout="fill"
                objectFit="cover"
                objectPosition="center"
                unoptimized
              />
            </a>
          </Link>
        </div>
        <div className="flex flex-col flex-1 w-full">
          <div className="flex w-full text-base mb-2">
            <Link href={`/store/${itemSlug}`}>
              <a>
                <span className={titleClass}>{itemName}</span>
              </a>
            </Link>
            <div className="flex flex-col justify-between ml-auto space-y-2 text-sm">
              <span className={priceClass}>
                {formatAmountWithSymbol({
                  amount: itemPrice?.regular,
                  currency: cart?.region?.currency_code,
                  tax: cart.region.tax_rate
                })}
              </span>
            </div>
          </div>
          {variant === 'advanced' && (
            <>
              {/* {!!category && <span className={metaClass}>{category}</span>}
              {!!size && <span className={metaClass}>Size: {size}</span>}
              {!!color && <span className={metaClass}>Color: {color}</span>} */}

              <div className="flex w-full overflow-hidden justify-start mt-auto mb-2">
                <Quantity
                  max={itemInventory}
                  value={itemQty}
                  handleRemove={handleRemove}
                  handleChange={handleChange}
                  increase={() => handleChangeQuantity(1)}
                  decrease={() => handleChangeQuantity(-1)}
                />
              </div>
            </>
          )}
        </div>
      </div>
      {variant === 'base' && (
        <Quantity
          max={itemInventory}
          value={itemQty}
          handleRemove={handleRemove}
          handleChange={handleChange}
          increase={() => handleChangeQuantity(1)}
          decrease={() => handleChangeQuantity(-1)}
        />
      )}
    </li>
  );
};

export default CartItem;
