import { useMemo } from 'react';
import { motion } from 'framer-motion';
import Link from 'next/link';
import Button from '@/components/common/Button';
import CartItem from './CartItem';
import { backdropVariant } from '@/app/constant/variant';
import IconClose from '../../../../public/images/icons/icon-close.svg';
/* composable */
import { cartGetters } from '@react-test-storefront/medusa';
import { useAppProvider } from '../../../context/AppProvider';
import { formatAmountWithSymbol } from '@/utils/price';

const SiderbarViewVariants = {
  show: {
    opacity: 1,
    x: '0%',
    transition: {
      duration: 0.2,
      type: 'tween',
    },
  },
  hide: {
    opacity: 0,
    x: '100%',
  },
};

const CartPopup = ({ onClose }) => {
  const { cartObject: { cart } } = useAppProvider();

  const cartItems = useMemo(() => cartGetters.getItems(cart), [cart]);
  const cartTotal = useMemo(() => cartGetters.getTotals(cart), [cart]);

  const hasItems = cartItems && cartItems.length > 0;

  return (
    <motion.div
      initial="hide"
      animate="show"
      exit="hide"
      variants={backdropVariant}
      className="fixed h-[100vh] top-0 bottom-0 left-0 right-0 z-1150"
    >
      <div className="absolute top-0 bottom-0 left-0 right-0 z-1150 bg-black bg-opacity-30" />

      <motion.div
        variants={SiderbarViewVariants}
        onClick={(e) => e.stopPropagation()}
        className="fixed h-[100vh] top-0 bottom-0 right-0 inset-y-0 z-150 bg-white flex w-full max-w-full outline-none md:w-auto overflow-x-hidden overflow-y-auto"
      >
        <div className="flex flex-col items-start justify-start w-full h-full min-h-full md:w-screen md:max-w-md">
          <div className="flex flex-col px-5 py-4">
            <button onClick={onClose}>
              <IconClose />
            </button>
          </div>
          <div className="w-full">
            {hasItems ? (
              <ul className="w-full">
                {cartItems.map((item) => (
                  <CartItem key={item.id} item={item} />
                ))}
              </ul>
            ) : (
              <div className="p-5">
                <h3>Your cart is empty</h3>
                <Link href={'/store'}>
                  <a className="text-blue-600">Go to Store</a>
                </Link>
              </div>
            )}
          </div>
          <div className="w-full px-5 py-4 mt-auto">
            <ul className="flex flex-col w-full mb-4 gap-y-2">
              <li className="flex">
                <span>Subtotal</span>
                <span className="ml-auto">
                  {formatAmountWithSymbol({ amount: cartTotal?.subtotal, currency: cart?.region?.currency_code })}
                </span>
              </li>
              <li className="flex">
                <span>Taxes</span>
                <span className="ml-auto">
                  {formatAmountWithSymbol({ amount: cartTotal?.taxes, currency: cart?.region?.currency_code })}
                </span>
              </li>
              <li className="flex pb-2">
                <span>Shipping</span>
                <span className="ml-auto ">
                  {formatAmountWithSymbol({ amount: cartTotal?.shipping, currency: cart?.region?.currency_code })}
                </span>
              </li>
              <li className="flex pt-4 font-semibold border-t border-t-gray-200">
                <span>Total</span>
                <span className="ml-auto">
                  {formatAmountWithSymbol({ amount: cartTotal?.total, currency: cart?.region?.currency_code })}
                </span>
              </li>
            </ul>
            <div>
              <Link href="/cart" passHref>
                <Button
                  width="100%"
                  variant="ghost"
                  className="mb-2"
                  Component="a"
                >
                  View Cart
                </Button>
              </Link>
              <Link href="/checkout" passHref>
                <Button width="100%" Component="a" disabled={!hasItems}>
                  Procede to Checkout
                </Button>
              </Link>
            </div>
          </div>
        </div>
      </motion.div>
    </motion.div>
  );
};

export default CartPopup;
