import React, { useEffect, useMemo, useState } from 'react';
import { useFormik } from 'formik';
import { useAppProvider } from '@/context/AppProvider';
import { cartGetters } from '@react-test-storefront/medusa';
import Button from '@/components/common/Button';
import { validationBilling, validationShipping } from '../validation';
import Shipping from './Shipping';
import CartSummary from '../../CartContainer/partials/CartSummary';
import Billing from './Billing';
import { SHIPPING } from '../constants';

const Address = ({ setComplete, setStep }) => {
  const [sameAddress, setSameAddress] = useState(true);
  const { getTotalItems } = cartGetters;
  const {
    shippingRegionObject: { shippingRegion, shippingRegionFunctions },
    shippingObject: { setShipping, shippingFunctions },
    billingObject: { setBilling, billingFunctions },
    cartObject: { cart }
  } = useAppProvider();

  const totalItems = useMemo(() => getTotalItems(cart), [cart]);

  const { save: saveBilling } = billingFunctions();
  const { save: saveShipping } = shippingFunctions();
  const { load: loadShippingRegion } = shippingRegionFunctions();

  useEffect(() => {
    if (!shippingRegion) {
      loadShippingRegion();
    }
  }, []); // eslint-disable-line

  useEffect(() => {
    setComplete((prev) => ({
      ...prev,
      address: 0,
      shipping: 0,
      payment: 0,
      done: 0,
    }));
  }, []); // eslint-disable-line

  const handleBillingAddress = useFormik({
    initialValues: {
      address1: '',
      address2: '',
      country: '',
      state: '',
      city: '',
      postalCode: '',
    },
    validationSchema: validationBilling,
    validateOnChange: true,
    onSubmit: async (values) => {
      setBilling(null);
      await saveBilling({ currentCart: cart, values });
      setComplete((prev) => ({ ...prev, address: 1 }));
      setStep(SHIPPING);
    },
  });

  const handleShippingAddress = useFormik({
    initialValues: {
      firstName: '',
      lastName: '',
      email: '',
      phone: '',
      address1: '',
      address2: '',
      country: '',
      state: '',
      city: '',
      postalCode: '',
    },
    validationSchema: validationShipping,
    validateOnChange: true,
    onSubmit: async (values) => {
      setShipping(null);

      await saveShipping({ currentCart: cart, values });
      if (sameAddress) {
        setBilling(null);
        await saveBilling({ currentCart: cart, values });
        setComplete((prev) => ({ ...prev, address: 1 }));
        setStep(SHIPPING);
      } else {
        handleBillingAddress.handleSubmit();
      }
    },
  });

  return (
    <div className="Address container mx-auto mt-12">
      <div className="grid grid-cols-12 lg:gap-x-16">
        <div className="col-span-12 md:col-span-8">
          <div className="w-full">
            <Shipping
              handleShippingAddress={handleShippingAddress}
            />
            <Billing
              handleBillingAddress={handleBillingAddress}
              sameAddress={sameAddress}
              setSameAddress={setSameAddress}
            />
            <div className="mt-12 mb-20">
              <Button disabled={totalItems === 0} type="submit" onClick={handleShippingAddress.handleSubmit} className="w-full lg:w-1/2">
                Continue to Billing
              </Button>
              {totalItems === 0 && <p>No items in your cart</p>}
            </div>
          </div>
        </div>
        <div className="col-span-12 mb-16 md:col-span-4">
          <CartSummary hideButton />
        </div>
      </div>
    </div>
  );
};

export default Address;
